<?php namespace Kliken\WpPlugin;

use Kliken\Exception\KK_Api_Exception;

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class KK_Plugin
{
	private static $instance = null;

	public static function get_instance() {
		if ( ! self::$instance ) {
			self::$instance = new KK_Plugin;
		}

		return self::$instance;
	}

	/**
	 * A starting point to the plugin initialization.
	 * - Add hooks to create
	 *   + Config/Settings menu
	 *   + Print the tracking code onto the footer
	 */
	public function init_hooks() {
		// Some sort of housekeeping before doing anything else
		add_action( 'admin_init', array( $this, 'house_keeping' ) );

		// Check if plugin's schema needs update
		add_action( 'admin_init', array( $this, 'update_schema' ) );

		// Add a settings menu to settings/config page of the plugin
		add_action( 'admin_menu', function() {
			add_menu_page(
		        __('Kliken All-In-One Marketing', KK_TEXT_DOMAIN),
		        __('Kliken Marketing', KK_TEXT_DOMAIN),
		        'manage_options',
		        KK_SETTING_PAGE,
		        array( '\Kliken\WpPlugin\KK_Plugin', 'config_page' ),
		        kk_get_base64_icon(),
		        '99.999'
		    );
		} );

		// Load assets for admin setting page(s)
		if ( kk_is_setting_page() ) {
			add_action( 'admin_enqueue_scripts', array( $this, 'load_admin_assets' ) );
		}

		// Tell user to continue with setting up the plugin
		if ( kk_no_tracking_code() ) {
			add_action( 'admin_notices', function() {
				// Show notice only if the current user has "manage_options" capability
				if (current_user_can( 'manage_options' )) {
					$message = sprintf(
						wp_kses(
							__( '<strong>Kliken All-In-One Marketing plugin is almost ready.</strong> You must <a href="%s">link your Kliken account</a> in order for it to work.', KK_TEXT_DOMAIN ),
							array(
								'strong' => array(),
								'a' => array( 'href' => array() )
							)
						), esc_url( kk_get_setting_page_link() )
					);

					echo Messages::add_warning( $message );
				}
			} );
		}

		// Add a handler to create account link via AJAX
		if ( is_admin() ) {
			add_action( 'wp_ajax_link_account', array( $this, 'ajax_link_account' ) );
			add_action( 'wp_ajax_reset_account', array( $this, 'ajax_reset_account' ) );
		}

		// Add tracking code to front-end footer if available
		if ( ( $tracking_script = kk_get_tracking_code() ) !== false ) {
			// This JS code containing a function call should be placed before the actual "sw.js" file call.
			add_action( 'wp_footer', function() use($tracking_script){
				echo
<<<"TRACKINGCODE"
<script type="text/javascript">
	var swPreRegister = function() { }
</script>
TRACKINGCODE;
				echo $tracking_script;
			} );
		}
	}

	/**
	 * Render the configuration/setting page.
	 * - The config page will include the process to smoothly sign the user(s) up with a Kliken account.
	 * - When the account is setup properly, it shows the user with a set of icons to quickly navigate to Kliken reports.
	 */
	public function config_page() {
		// Get KK_Plugin instance because this function might get called from closure and not really have access to $this
		$inc = \Kliken\WpPlugin\KK_Plugin::get_instance();

		$setupFinished = false;
		$args = array();

		// Include the correct view files
		if ( kk_no_tracking_code() ) {
			// If has invitation code, try to get authentication info from it
			if ( kk_for_invitation() ) {
				// Do API call
				$account_info = KK_Api::get_account_from_invitation(
					get_option( KK_OPTION_NAME_API_TOKEN ),
					get_option( KK_OPTION_NAME_INVITATION_CODE )
				);

				// Success
				if ( $account_info->AccountId !== null ) {
					$inc->setup_plugin_data( $account_info->AccountId );

					if ( file_exists( KK_PLUGIN_DIR . 'config.php' ) ) {
						unlink( KK_PLUGIN_DIR . 'config.php' );
					}

					$setupFinished = true;
					$args = array(
						'reload' => true
					);
				}
			}
		} else {
			$setupFinished = true;
		}

		if ( true === $setupFinished ) {
			$inc->view( 'settings', $args );
		} else {
			$current_user = wp_get_current_user();

			$args = array(
				'kk_signup_url' => KK_HOST . 'auth/newaccount-wpkliken.aspx?'
					. 'aff=' . urlencode( strtolower( get_option( KK_OPTION_NAME_AFFILIATE_ID, '' ) ) )
					. '&u=' . urlencode( home_url() )                     // site url
					. '&n=' . urlencode( $current_user->display_name )    // name
					. '&e=' . urlencode( $current_user->user_email ),     // email
				'kk_login_url' => KK_HOST . 'plugins/wordpress/Default.aspx?login=true&brand=Kliken'
 			);

			$inc->view( 'config' , $args );
		}
	}

	/**
	 * @param string $name Name of the view to be loaded. The file must locates in "views" folder.
	 * @param array  $data Data to be passed to the view
	 */
	public function view( $name, array $data = array() ) {
		$file = KK_PLUGIN_DIR . "views/{$name}.php";
		if ( true === file_exists( $file ) ) {
			// Include the template file
			include $file;
		}
	}

	/**
	 * Load some common assets for (only) admin page
	 */
	public function load_admin_assets() {
		wp_enqueue_style( 'kk-style', KK_PLUGIN_URL . 'assets/kk-style.css' );

		wp_enqueue_script( 'jquery-core' ); // In case there's no jQuery loaded yet
	}

	/**
	 * Perform link account upon receiving an AJAX request. CSRF protection enabled!
	 */
	public function ajax_link_account() {
		// Security check
		check_ajax_referer( 'kk-link-account-nonce', 'kkAjaxNonce' );

		$api_token = sanitize_text_field( $_POST['apiToken'] );
		$user_token = sanitize_text_field( $_POST['userToken'] );

		$error = '';

		// Check if the tokens are working by making request to REST API
		if ( $api_token && $user_token ) {
			try {
				$account_info = KK_Api::get_account( $api_token, $user_token );

				if ( isset( $account_info->accountNumber ) ) {
					self::setup_plugin_data( $account_info->accountNumber );
				}
			} catch ( KK_Api_Exception $e ) {
				$error = sprintf( __( 'SW API call unsuccessful! %s', KK_TEXT_DOMAIN ), $e->getMessage() );
			} catch ( \Exception $e ) {
				// Think of something to log the catchable errors
				$error = sprintf( __( 'Caught exception: %s', KK_TEXT_DOMAIN ), $e->getMessage() );
			}
		}

		echo json_encode(array(
			'error' => $error
		));

		wp_die(); // For ajax to return immediately with the JSON
	}

	public function ajax_reset_account() {
		// Security check
		check_ajax_referer( 'kk-reset-account-nonce', 'kkAjaxNonce' );

		// Check passed, so just remove database-stored value
		kk_plugin_db_cleanup( false );

		echo json_encode( array( 'success' => true ) );

		wp_die();
	}

	public function house_keeping() {
		// Set config information if available
		if ( file_exists( KK_PLUGIN_DIR . 'config.php' ) ) {
			$kk_affiliate_id = '';
			$kk_invitation_code = '';
			$kk_account_token = '';

			include KK_PLUGIN_DIR . 'config.php';

			if ( $kk_affiliate_id ) update_option( KK_OPTION_NAME_AFFILIATE_ID, $kk_affiliate_id );
			if ( $kk_invitation_code ) update_option( KK_OPTION_NAME_INVITATION_CODE, $kk_invitation_code);
			if ( $kk_account_token ) update_option( KK_OPTION_NAME_API_TOKEN, $kk_account_token);
		}

		// Set database schema version if there is not one yet
		if ( false === get_option( KK_OPTION_NAME_SCHEMA_VERSION ) ) {
			add_option( KK_OPTION_NAME_SCHEMA_VERSION, KK_PLUGIN_SCHEMA_VERSION );
		}
	}

	public function update_schema() {
		if ( ! is_admin() || ! defined( 'KK_PLUGIN_SCHEMA_VERSION' ) ) return;

		// Get current schema version. Default to 1. We might not have this before.
		$current_schema_version = get_option( KK_OPTION_NAME_SCHEMA_VERSION, '1' );

		if ( version_compare( $current_schema_version, KK_PLUGIN_SCHEMA_VERSION, '<' )) {
			$updater = new KK_Updater();

			for ( $v = $current_schema_version; $v < KK_PLUGIN_SCHEMA_VERSION; $v++ ) {
				// Check if the updater function exists. In any loop, if it's not, break immediately!
				$method_name = "UpdateFrom{$v}To" . ($v + 1);
				if ( ! method_exists( $updater, $method_name ) ) break;

				if ( $updater->{$method_name}() ) {
					// Update the current schema to the latest (code) version
					update_option( KK_OPTION_NAME_SCHEMA_VERSION, $v + 1 );
				}
			}
		}
	}

	private function setup_plugin_data( $account_id )
	{
		$tracking_script =
<<<"TRACKINGSCRIPT"
	<script type="text/javascript">
		var scheme = ( "https:" === document.location.protocol ) ? "https://" : "http://";
		document.write(decodeURI("%3Cscript src='" + scheme + "analytics.sitewit.com/v3/{$account_id}/sw.js' type='text/javascript'%3E%3C/script%3E"));
	</script>
TRACKINGSCRIPT;

		update_option( KK_OPTION_NAME_TRACKING_SCRIPT, $tracking_script );

		// Done. We don't need account/api token anymore.
		delete_option( KK_OPTION_NAME_API_TOKEN );
	}
}
